//This example code is in the Public Domain (or CC0 licensed, at your option.)
//By Evandro Copercini - 2018
// 
//This example creates a bridge between Serial and Classical Bluetooth (SPP)
//and also demonstrate that SerialBT have the same functionalities of a normal Serial

#include "BluetoothSerial.h"
#include <math.h>

#if !defined(CONFIG_BT_ENABLED) || !defined(CONFIG_BLUEDROID_ENABLED)
#error Bluetooth is not enabled! Please run `make menuconfig` to and enable it 
#endif

 
#define TPIN 35
#define BatPIN 34

#define FanPIN 23

#define PWMPIN 4
#define PWM_PERIOD 10000 // 10000 ms = 0.1 Hz


const double r_ser = 100;   //98.1 kilo Om
const double A = 0.0009119686;    //0.0006443759;  // staro 0.0006443759      
const double B = 0.0001922298;    //0.0002277425; // staro 0.0002277425
const double C = 0.0000001668904; //0.00000005744251; // staro 0.00000005744251

double analog_value = 0;
double r_ntc = 0;
double logR = 0;
double temp = 0;

double bat_analog = 0;
double bat_voltage = 0;
double bat_percent = 0;
char state = 1;

uint PWM_duty_cycle_percent = 100;   // 0, 50, 100

uint previousTime = 0;


BluetoothSerial BT;

void setup() {

  pinMode(TPIN, INPUT);
  pinMode(BatPIN, INPUT);
  pinMode(PWMPIN, OUTPUT);
  pinMode(FanPIN, OUTPUT);

  digitalWrite(FanPIN, HIGH);

  Serial.begin(9600);
  BT.begin("ESP32Peltier"); // BT ime uredjaja
  Serial.println("The device started, now you can pair it with bluetooth!");

}

void loop() {

  doTemp();

  doBat();

  doPWM();

  doBT();

}

void doBT() {
  if (BT.available()) {  // Проверава да ли постоји примљен податак преко Bluetooth-а
    char receivedChar = BT.read();  // Чита примљени карактер

    // Ако је примљен 'B'
    if (receivedChar == 'B') {

      digitalWrite(PWMPIN, LOW); // ugasi potrosac
      digitalWrite(FanPIN, LOW);

      delay(100); // sacekaj malo
      doBat();

      Serial.println("Primljeno 'B'");
      BT.print("Primljeno 'B' ---- Baterija je: ");
      BT.print(bat_percent);
      BT.println("%");
    
      digitalWrite(FanPIN, HIGH); // upali ventilator

      if(bat_percent < 5) {
        Serial.println("!!! PRAZNA BATERIJA GASIM SE !!!");
        BT.print("!!! PRAZNA BATERIJA GASIM SE!!!");
        receivedChar = '3'; // kako bi se ugasio
      }
      else if(bat_percent < 10) {
        Serial.println("!!! PRAZNA BATERIJA !!!");
        BT.print("!!! PRAZNA BATERIJA !!!");
      }

    }

    // Ако је примљен '1'
    if (receivedChar == '1') {
      Serial.println("Primljeno '1'");
      BT.print("Primljeno '1' ---- MIN");
      PWM_duty_cycle_percent = 50;
      digitalWrite(FanPIN, HIGH);
      state = 1;
    }

    // Ако је примљен '2'
    else if (receivedChar == '2') {
      Serial.println("Primljeno '2'");
      BT.print("Primljeno '2' ---- MAX");
      PWM_duty_cycle_percent = 100;
      digitalWrite(FanPIN, HIGH);
      state = 1;
    }

    // Ако је примљен '3'
    else if (receivedChar == '3') {
      Serial.println("Primljeno '3'");
      BT.print("Primljeno '3' ---- Gasim hladjenje");
      PWM_duty_cycle_percent = 0;
      digitalWrite(FanPIN, LOW);
      state = 0;
    }

    // Ако је примљен 'T'
    else if (receivedChar == 'T') {
      Serial.println("Primljeno 'T'");
      BT.print("Primljeno 'T' ---- Temperatura NTCa je: ");
      BT.println(temp);
    }
  }
}

void doPWM() {

  if (PWM_duty_cycle_percent == 100) {
    digitalWrite(PWMPIN, HIGH);
  }

  else if (PWM_duty_cycle_percent == 50) {

    if(millis() - previousTime > PWM_PERIOD / 2) {
      digitalWrite(PWMPIN, HIGH);        
    }
    else {
      digitalWrite(PWMPIN, LOW); 
    }
  }

  else {
    digitalWrite(PWMPIN, LOW);
  }

  if (millis() - previousTime > PWM_PERIOD) {
      previousTime = millis();
    }

}

void doBat() {
  
  bat_analog = analogRead(BatPIN);
  bat_voltage = bat_analog * (3.3 / 4095.0); // prebacuje u volte
  bat_voltage *= 1000;  // ponistava trecu decimalu
  bat_percent = map((long)bat_voltage, 2280, 2720, 0, 100); // pretvara napon baterije u procente, 3.7 = 0%, 4.2 = 100%
  //empirijski, 2410 za 3.7,  2800 za 4.2 V, 
  //empirijski, kad je ugasen peltier i fan, 3.7 ~= 2300  2700 ~= 4.2, ide +- 20 da ne bi prelazilo 100% i padalo ispod 0%
  //zbog naponskog razdelnika 2.517 = 3.6 a 2.857 = 4.2

}

void doTemp() {
  
  analog_value = analogRead(TPIN);

  r_ntc = r_ser / (4095/analog_value - 1);
  r_ntc *= 1000;    // prebacivanje iz kilo oma u ome 
 
  logR = log(r_ntc);

  temp = A + B * logR + C * pow(logR, 3);
  temp = 1/temp;
  temp -= 273.15;   // prebacivanje u stepene Celzijusa
  
  Serial.print("T=");
  Serial.println(temp, 1);

}
